#! /usr/bin/python3
#
#################################################################
## Copyright (c) 2015 Norbert S. <junky-zs@gmx.de>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
#################################################################
#################################################################

import socketserver, socket
import threading, queue
import time, os

__author__  = "zs"
__status__  = "draft"
__version__ = "0.1"
__date__    = "12.2018"


#---------------------------------------------------------------------------
#   priority-queue related stuff
#---------------------------------------------------------------------------
#
# definitions for using priority-queues
#   lower values are defined as higher priorities
#
#   remark:
#     these values are currently not realy used, but will be in the future
#
INT_PRIO_LOW    = 40
INT_PRIO_MEDIUM = 20
INT_PRIO_HIGH   = 10
INT_PRIO_URGEND =  0


class cfileread(threading.Thread):
    """class 'cfileread' for reading binfile-data from already
       opened file
    """
    global _ClientHandler

    def __init__(self, hfile):
        threading.Thread.__init__(self)
        self.__threadrun=True
        self.__hfile=hfile
        self.__queueprio=INT_PRIO_MEDIUM

    def __del__(self):
        self.stop()

    def run(self):
        print("cfileread() ;thread start")
        while self.__threadrun:
            if _ClientHandler.get_clientcounter() > 0:
                try:
                    value=self.__hfile.read(5)
                except:
                    print("cfileread();Error;couldn't use/read required file")
                    self.__threadrun=False
                try:
                    for clientQueue in _ClientHandler._txqueue.items():
                        #put file readvalue in any connected client-queue
                        #  clientQueue[0]:=Client-ID; clientQueue[1]:=queue
                        #    put value into queue
                        clientQueue[1].put(value)
                except:
                   print("Client-ID:{0};cfileread();couldn't write to queue".format(clientQueue[0]))
            else:
                time.sleep(0.5)

        print("cfileread() ;thread end")

    def stop(self):
        self.__threadrun=False

class cfilewrite(threading.Thread):
    """class 'cfilewrite' for writing socket received command-data to already
       opened txlogfile
    """
    global _ClientHandler

    def __init__(self, hfile):
        threading.Thread.__init__(self)
        self.__threadrun=True
        self.__hfile=hfile
        self.__queueprio=INT_PRIO_MEDIUM


    def __del__(self):
        self.stop()

    def run(self):
        """bytes are read from queue, searched for start-tag '#' and length of
            following bytes. Then all bytes are written to comport
            # following queue-message_structur is supported:
            #   tag  length   class   detail  option  databytes.....
            #    #   <size>   ! or ?   d       o       bytes.....
            #      size := amount of databytes including class, detail and option but without starttag
            #
        """
        print("cfilewrite();thread start")
        while self.__threadrun:
            if _ClientHandler.get_clientcounter() > 0:
                #preset local buffers
                self.__starttag_found=False
                self.__length        =0
                readbuffer=[0,]
                try:
                    # 1. search for start-tag '#'
                    while not self.__starttag_found and self.__threadrun and (_ClientHandler.get_clientcounter() > 0):
                        #get comport writevalues from any connected client-queue
                        for clientQueue in _ClientHandler._rxqueue.items():
                            try:
                                # use get() with timeout
                                readbuffer=clientQueue[1].get(timeout=0.2)
                                clientQueue[1].task_done()
                                if readbuffer[0] == 0x23:
                                    self.__starttag_found=True
                                    break
                            except:
                                # timeout occured-> no exception
                                pass

                    # 2. now get msg-length from stream (over all length including headerbytes but without starttag)
                    if self.__starttag_found and len(readbuffer) > 1:
                        self.__length=readbuffer[1]

                    # 3. now read rest of msg-headerbytes from stream and set resulting payload-length
                    if self.__starttag_found and len(readbuffer) > 4:
                        msg_class =readbuffer[2]
                        msg_detail=readbuffer[3]
                        msg_option=readbuffer[4]
                        if self.__length >= 3:
                            self.__length -= 3
                        else:
                            self.__length = 0
                    else:
                        self.__starttag_found=False

                    self.__msgbytes=[]
                    try:
                        if self.__starttag_found and self.__length > 0:
                            self.__msgbytes.extend(readbuffer[5:self.__length+5])
                    except:
                        raise

                    # 4. send message-class/detail/option and (data-bytes if available) to transceiver_if
                    if self.__starttag_found:
                        try:
                            print("ClientID:{0};data:{1};msg_class:{2};msg_detail:{3};msg_option:{4}".format(clientQueue[0], self.__msgbytes, msg_class, msg_detail, msg_option))
                        except:
                            print("Client-ID:{0};cfilewrite();couldn't write to file".format(clientQueue[0]))
                            self.__threadrun=False
                            self.__starttag_found=False
                except:
                    print("Client-ID:{0};cfilewrite();couldn't read from queue".format(clientQueue[0]))
            else:
                time.sleep(0.2)

        print("cfilewrite();thread end")

    def stop(self):
        self.__threadrun=False


class cht_fileIO_if(threading.Thread):
    """class 'cht_fileIO_if' handles the byte-transfer to socket-IF.
       The used file must be accessable and is used exclusive one time.
       All file-data are written to queue(s),(unique for every socket-client)
         this is handled with class: cfileread
       All transmitted serial data are read from queue(s),(unique for every socket-client)
         this is handled with class: cfilewrite
    """
    global _ClientHandler
    def __init__(self, binlogfile="./binlogfile", txbytes_logfile="./txbytes.txt"):
        threading.Thread.__init__(self)
        self.__binlogfile=binlogfile
        self.__txbytesfile=txbytes_logfile
        self.__h_logfile=None
        self.__h_txlogfile=None
        self.__threadrun=True

    def __del__(self):
        self.stop()
        if self.__h_logfile != None:
            self.__h_logfile.close()
        if self.__h_txlogfile != None:
            self.__h_txlogfile.close()

    def run(self):
        #open bin-file for for reading HT-data
        self.__binlogfile="./"+str(input("Eingabe logfile-name-->:"))
        try:
            self.__h_logfile = open(self.__binlogfile,"rb")
        except:
            print("cht_fileIO_if();Error;couldn't open requested file:{0}".format(self.__binlogfile))
            self.__threadrun=False
            raise

        try:
            self.__h_txlogfile = open(self.__txbytesfile,"w")
        except:
            print("cht_fileIO_if();Error;couldn't open tx log-file:{0}".format(self.__txbytesfile))
            self.__threadrun=False
            raise

        self.__comtx_thread=cfilewrite(self.__h_txlogfile)
        self.__comtx_thread.start()
        self.__comrx_thread=cfileread(self.__h_logfile)
        self.__comrx_thread.start()

        while self.__threadrun:
            time.sleep(1)

    def stop(self):
        self.__com_txthread.stop()
        self.__com_rxthread.stop()
        self.__threadrun=False


class csocketsendThread(threading.Thread):
    """class 'csocketsendThread' used for sending data from queue to
       already connected socket
    """
    def __init__(self, request, queue):
        threading.Thread.__init__(self)
        self._queue  =queue
        self._request=request
        self.__threadrun=True
        self.__queueprio=INT_PRIO_MEDIUM

    def __del__(self):
        self.__threadrun=False
        #clear queue
        while self._queue.qsize() > 0:
            self._queue.get_nowait()

    def run(self):
        print("csocketsendThread(); socket.send thread start")
        self._tx=None
        while self.__threadrun==True:
            try:
                # get queue-value in blocking mode
                self._tx=self._queue.get(True)
                self._queue.task_done()
            except:
                self.__threadrun=False
                print("csocketsendThread();Error on queue.get()")
                raise

            try:
                self._request.sendall(bytes(self._tx))
            except:
                self.__threadrun=False
                print("csocketsendThread();Error on socket.send")
                raise

        print("csocketsendThread(); socket.send thread terminated")

    def stop(self):
        self.__threadrun=False


class cClientHandling(threading.Thread):
    """class 'cClientHandling' used for add and remove clients to / from queues and
       threads. logging-methods are available for different logging-levels.
    """
    def __init__(self):
        threading.Thread.__init__(self)

        self._indexcounter=0
        self._clientcounter=0
        self._lock=threading.Lock()
        self._rxqueue={}
        self._txqueue={}
        self._thread={}

    def inc_indexcounter(self):
        self._lock.acquire()
        self._indexcounter+=1
        self._lock.release()

    def get_indexcounter(self):
        self._lock.acquire()
        counter=self._indexcounter
        self._lock.release()
        return counter

    def inc_clientcounter(self):
        self._lock.acquire()
        self._clientcounter+=1
        self._lock.release()

    def dec_clientcounter(self):
        self._lock.acquire()
        self._clientcounter-=1
        self._lock.release()

    def get_clientcounter(self):
        self._lock.acquire()
        counter=self._clientcounter
        self._lock.release()
        return counter

    def add_client(self, clientID, request):
        self._rxqueue.update({clientID:queue.Queue()})
        self._txqueue.update({clientID:queue.Queue()})

        txThread=csocketsendThread(request, self._txqueue.get(clientID))
        self._thread.update({clientID:txThread})
        txThread.start()
        print("Client-ID:{0}; added; number of clients:{1}".format(clientID, self._clientcounter))

    def remove_client(self, clientID):
        txThread=self._thread.pop(clientID)
        txThread.stop()
        queue=self._rxqueue.pop(clientID)
        while queue.qsize() > 0:
            queue.get_nowait()
        queue=self._txqueue.pop(clientID)
        while queue.qsize() > 0:
            queue.get_nowait()
        print("Client-ID:{0}; removed; number of clients:{1}".format(clientID, self._clientcounter))


class cht_RequestHandler(socketserver.BaseRequestHandler):
    """
    """
    global _ClientHandler

    def handle(self):
        self._rx=None
        self._client_devicetype=None
        self.__queueprio=INT_PRIO_HIGH
        try:
            addrc, portc = self.client_address
            addrs, ports = self.server.server_address
            print("Client-ID:{0}; {1} connected".format(self._myownID, (addrc, portc)))
            #print("Server   :{0}".format((addrs,ports)))
        except:
            addrc, portc = self.client_address
            print("Client-ID:{0}; {1} No connection possible".format(self._myownID, (addrc, portc)))
            raise
        # wait for client registration
        self.__waitfor_client_register()
        # add client and start threads
        _ClientHandler.add_client(self._myownID, self.request)
        self._rxqueue=_ClientHandler._rxqueue.get(self._myownID)

        #print("Client-ID:{0}; cht_RequestHandler(); socket.receive thread start".format(self._myownID))
        while True:
            try:
                self._rx=self.request.recv(60)
            except:
                print("Client-ID:{0}; {1} disconnected".format(self._myownID, (addrc, portc)))
                break
            if self._rx:
                # put socket-data in queue
                self._rxqueue.put(self._rx)
                print("Client-ID:{0}; recv:{1}".format(self._myownID, self._rx))
            else:
                print("Client-ID:{0}; {1} disconnected".format(self._myownID, (addrc, portc)))
                break


    def __waitfor_client_register(self):
        self.request.settimeout(5)
        try:
            devicetypetmp=self.request.recv(20)
            self._client_devicetype = devicetypetmp.decode('utf-8')
            print("Client-ID:{0}; register(); got devicetype:{1}".format(self._myownID,self._client_devicetype))
            #send client-ID to client
            sendtemp=str(self._myownID)
            self.request.sendall(sendtemp.encode("utf-8"))
        except socket.timeout:
            print("Client-ID:{0}; Timeout occured, no devicetype was send".format(self._myownID))
            raise
        except socket.error as e:
            # Something else happened, handle error, exit, etc.
            print("Client-ID:{0}; error '{1}' on socket.recv or socket.send".format(self._myownID, e))
            raise
        except Exception as e:
            print("Client-ID:{0}; unkown error '{1}'".format(self._myownID,e))
            raise
        finally:
            self.request.settimeout(None)


    def setup(self):
        _ClientHandler.inc_indexcounter()
        _ClientHandler.inc_clientcounter()
        self._myownID=_ClientHandler.get_indexcounter()

    def finish(self):
        _ClientHandler.dec_clientcounter()
        _ClientHandler.remove_client(self._myownID)


class cht_f2s_daemon(threading.Thread):
    """class 'cht_f2s_daemon', create object using this as server-daemon.
       ip_address and port_number are set with fixed values.
    """
    global _ClientHandler

    def __init__(self):
        threading.Thread.__init__(self)
        self._ip_address="localhost"
        self._port_number=10000
        self._server=None

        #read configfile
        try:

            global _ClientHandler
            _ClientHandler=cClientHandling()
            print("----------------------")
        except:
            print("cht_f2s_daemon();error can't get/set configurationvalues")
            raise

    def __del__(self):
        pass

    def run(self):
        print("cht_f2s_daemon start as server:'{0}';port:'{1}'".format(self._ip_address, self._port_number))

        try:
            fileIO_if = cht_fileIO_if()
            fileIO_if.setDaemon(True)
            fileIO_if.start()
        except:
            print("cht_f2s_daemon() terminated; couldn't start fileIO_IF")
            print("---------------------------")
            raise

        try:
            self._server=socketserver.ThreadingTCPServer((self._ip_address, self._port_number), cht_RequestHandler)
            self._server.serve_forever()
            print("cht_f2s_daemon terminated")
            print("---------------------------")
            raise
        except:
            print("cht_f2s_daemon terminated")
            print("---------------------------")
            raise
#--- class cht_f2s_proxy_if end ---#

class cht_socket_client():
    """class 'cht_socket_client', create object to connect to server
       using ip_address and port_number from configuration-file.
    """
    def __init__(self):
        self._ip_address ="localhost"
        self._port_number=10000
        self._clientID   =0
        self._devicetype="MODEM"

        self._socket=None
        #read configfile

        try:
            self._port_number = 10000

            print("----------------------")
            print("cht_socket_client init")

        except:
            print("cht_socket_client();error can't set configurationvalues")
            raise

        try:
            self._socket=socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            self._socket.connect((self._ip_address, self._port_number))
            print("connected to server:'{0}';port:'{1}'".format(self._ip_address, self._port_number))
        except:
            print("cht_socket_client.__init__();error:can't connect to socket")
            raise

        #send registration to proxy-server and receive client-related informations from server
        self.__registration();
        print("Client-ID:{0}; registered with devicetype:'{1}'".format(self._clientID, self._devicetype))

    def __del__(self):
        if self._socket != None:
            self._socket.close()
            print("Client-ID:{0}; socket closed".format(self._clientID))

    def __registration(self):
        try:
            # send devicetype to server
            devicetype=bytearray(self._devicetype.encode("utf-8"))
            self.write(devicetype)

            # read answer from server (client-ID) and store it
            self._clientID=self._socket.recv(10).decode('utf-8')
            ## only for test## print("client got ID:{0}".format(self._clientID))
        except:
            self._socket.close()
            print("cht_socket_client.__registration();error:can't register to master")
            raise

    def run(self):
        try:
            print("Client-ID:{0}; cht_socket_client run".format(self._clientID))
            while True:
                antwort=self._socket.recv(1)
                print("{0}".format(antwort))
        except:
            self._socket.close()
            print ("Client-ID:{0} ; cht_socket_client.run(); error on socket.recv".format(self._clientID))
            raise

    def close(self):
        if self._socket != None:
            self._socket.close()
            self._socket = None
            print("Client-ID:{0}; socket closed".format(self._clientID))

    def read(self, size=1):
        """Read size bytes from the connected socket. It will block
           until the requested number of bytes are read.
        """
        if self._socket==None:
            raise ("Client-ID:{0}; cht_socket_client.read(); error:socket not initialised".format(self._clientID))
        read=bytearray()
        while len(read) < size:
            try:
                buffer=self._socket.recv(size)
            except:
                self._socket.close()
                print("Client-ID:{0}; cht_socket_client.read(); error on socket.recv".format(self._clientID))
                raise

            if not buffer:
                self._socket.close()
                print("Client-ID:{0}; cht_socket_client.read(); peer closed socket".format(self._clientID))
                raise
            else:
                read.extend(buffer)

        return bytes(read)

    def write(self, data):
        """write data to connected socket. It will block
           until all data is written.
        """
        if self._socket==None:
            print("Client-ID:{0}; cht_socket_client.write(); socket not initialised".format(self._clientID))
            raise
        try:
            self._socket.sendall(bytes(data))
        except:
            print("Client-ID:{0}; cht_socket_client.write(); error on socket.sendall".format(self._clientID))
            raise

#--- class cht_socket_client end ---#


################################################

if __name__ == "__main__":
    import time
    # set flag to True for client-check
    ClientCheckFlag = False

    print("----- do some daemon-server-checks with connected client -----")
    print("   -- start socket.server --")
    ht_f2s_proxy=cht_f2s_daemon()
    ht_f2s_proxy.start()
    if ClientCheckFlag == True:
        time.sleep(1)
        print("   -- start socket.client --")
        client=cht_socket_client()
        time.sleep(1)
        # data-stream send to ht_transceifer_if: '#' ,'!' ,'S',option, length(payload)
        #   data = [0x23,0x21,0x53,0,0]
        # data-stream to be send on socket : '#' , <length> ,'!' ,'S',option
        print("     -- write 5 bytes to proxy-server for test --")
        data = [0x23,3,0x21,0x53,0]
        client.write(data)
        time.sleep(1)

        print("send commands to IF --")
        # data-stream send to ht_transceifer_if: '#' ,'!' ,'M', 0xF0, 0
        #   data = [0x23,0x21,0x53,0xF0,0]
        # data-stream to be send on socket : '#' , <length> ,'!' ,'M',0xF0
        reset_command = [0x23,3,0x21,0x4D,0xF0]

        loops=3

        while loops > 0:
            print("count down:{0}".format(loops))
            client.write(reset_command)
            time.sleep(2)
            print("Rx:{0}".format(client.read(5)))
            time.sleep(5)
            loops -= 1

        client.close()
        time.sleep(1)
        print("server still running and waiting for clients!")
        time.sleep(1)
        print("start second client!")
        client2=cht_socket_client()
        time.sleep(1)
        client2.write(data)
        time.sleep(5)
        print("stopp second client!")
        client2.close()
        print("server still running and waiting for clients!")
    else:
        while True:
            time.sleep(1)
